<?php

namespace App\Console\Commands;

use App\Helpers\Constants;
use App\Models\CrawlerListen;
use App\Models\Crontjob;
use App\Repositories\TrackRepository;
use Carbon\Carbon;
use Illuminate\Console\Command;

class CrawlerTrackListenCommand extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'crawler:get-listen {from_page?} {--limit=}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Lấy lượt nghe của bài hát ở các hệ thống dựa vào tham số truyền vào';

    protected $trackRepository;

    /**
     * Create a new command instance.
     *
     * CrawlerTrackListenCommand constructor.
     * @param TrackRepository $trackRepository
     */
    public function __construct(TrackRepository $trackRepository)
    {
        parent::__construct();
        ini_set('max_execution_time', -1);
        $this->trackRepository = $trackRepository;
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        $from_page = $this->argument('from_page');
        $limit = $this->option('limit');

        if (empty($from_page) || !in_array($from_page, ['zing', 'nct', 'nhacvn', 'keeng'])) {
            $from_page = $this->choice('Chọn trang nhạc ', ['zing', 'nct', 'nhacvn', 'keeng']);
        }

        if (empty($limit)) {
            $limit = Constants::CRAWLER_LISTEN_LIMIT;
        } else {
            $limit = -1;
        }

        echo "\n Bắt đầu xử lý lấy lượt nghe từ: " . $from_page;

        $crawlered = $this->trackRepository->getCrawlerListenInserted($from_page, false);

        echo "\n Tổng số bài hát đã lấy lượt nghe hôm nay: " . count($crawlered);

        // Lấy blacklist
        $blacklist_listen = $this->trackRepository->getBacklistListen($from_page);

        // Lấy mảng bài hát theo từng hệ thống nhạc
        $inserted = $this->trackRepository->getTrackBySource($from_page, $crawlered, $limit, false, $blacklist_listen);
        $total = count($inserted);

        echo "\n Tổng số bài hát cần lấy lượt nghe: " . $total;

        if ($total > 0) {
            $data_listen = [];
            foreach ($inserted as $key => $track) {
                $listen = $this->privateGetListen($from_page, $track);
                if ($listen > 0) {
                    $data_listen[$track['id']] = $listen;
                }
                echo "\n " . ($key + 1) . "/" . $total . ". " . $track['title'] . " (Track ID= " . $track['id'] . "): " . $listen;
            }

            if (count($data_listen) > 0) {
                foreach ($data_listen as $track_id => $listen ) {
                    switch ($from_page) {
                        case 'zing':
                            CrawlerListen::updateOrCreate(
                                ['track_id' => $track_id],
                                ['today_zing' => $listen, 'zing_crawler_at' => Carbon::now()]
                            );
                            break;
                        case 'nct':
                            CrawlerListen::updateOrCreate(
                                ['track_id' => $track_id],
                                ['today_nct' => $listen, 'nct_crawler_at' => Carbon::now()]
                            );
                            break;
                        case 'keeng':
                            CrawlerListen::updateOrCreate(
                                ['track_id' => $track_id],
                                ['today_keeng' => $listen, 'keeng_crawler_at' => Carbon::now()]
                            );
                            break;
                    }
                }
            }
        }

        echo "\n Lấy số lượt nghe kết thúc.";

        // Ghi log xem có gọi không
        Crontjob::create(['message' => 'CrawlerTrackListenCommand is called ('.$from_page . ')']);
    }

    /**
     * Hàm lấy lượt nghe thông qua api của các trang nhạc
     *
     * @param $src
     * @param $track
     * @return int
     */
    private function privateGetListen($src, $track)
    {
        $listen = 0;

        switch ($src) {
            case 'zing':
                $url = getZingUrlGetCounter($track['id_zing']);
                $curl = cURL($url);
                $response = json_decode($curl);

                if (isset($response->err) && $response->err == 0) {
                    if (isset($response->data) && isset($response->data->total) && $response->data->total > 0) {
                        $listen = $response->data->total;
                    }
                }

                break;
            case 'nct':
                /*$url = getNctUrlGetCounter($track['id_nct']);
                $curl = cURL($url);
                $response = json_decode($curl);

                if (isset($response->error_code) && $response->error_code == 0) {
                    if (isset($response->data) && isset($response->data->songs) && $response->data->songs->{$track['id_nct']} > 0) {
                        $listen = $response->data->songs->{$track['id_nct']};
                    }
                }*/

                /* Tính lượt nghe lấy data từ biểu đồ lượt nghe */
                $url = getNctUrlGetCounterForChart($track['id_nct']);
                $curl = cURL($url);
                $response = json_decode($curl);

                if (isset($response->status) && $response->status == 'Success') {
                    if (isset($response->dailyChart) && isset($response->dateChart) && isset($response->cumuChart) &&
                        is_array($response->dailyChart) && is_array($response->dateChart) && is_array($response->cumuChart))
                    {
                        /*foreach ($response->dailyChart as $key => $val) {
                            $listen[$key]['track_id'] = $track['id'];
                            $listen[$key]['daily_chart'] = $val;
                            $listen[$key]['date_chart'] = date('Y-m-d', strtotime(str_replace('/', '-', $response->dateChart[$key])));
                            $listen[$key]['cumu_chart'] = $response->cumuChart[$key];
                        }*/

                        //echo "\n " . end($response->cumuChart) . " | " . end($response->dailyChart) . " : " . $track['id_nct'];

                        $listen = end($response->cumuChart) + end($response->dailyChart);
                    }
                }

                break;
            case 'keeng':
                $url = getKeengUrlGetCounter($track['id_keeng']);
                $curl = cURL($url);
                $response = json_decode($curl);

                if (isset($response->data)) {
                    if (isset($response->data->listen_no) && $response->data->listen_no > 0) {
                        $listen = $response->data->listen_no;
                    }
                }

                break;
        }

        return $listen;
    }
}
